// 重组depMap的内容，产生自己所需要的代码

// 根据现有的 depMap，获取新的depMap，依据type
// 有三种使用场景：
//  1. 生成 .xtpl 后缀的
//  2. 入口文件生成 .native.cmd 后缀的
//  3. 入口文件生成 .web.cmd 后缀的

const _ = require('lodash');

var Debug = require('debug');
var debug = Debug('deps');
var debugMini = Debug('deps:mini');
var debugError = Debug('deps:error');
var debugExtra = Debug('deps:extra');

var util = require('./util');

var builtInList = util.getBuiltInMods(); // 内置native的白名单模块
debugMini('=====> 内置native的白名单模块：', builtInList);


// 原始 depMap 的遍历生成，兼容之前的代码
function processOrigin(config) {

  // 参数解构
  var depMap = config.depMap;
  var type = config.type;
  var mainEntryModName = config.mainEntryModName;
  var disableBuiltIn = config.disableBuiltIn;
  var disableXtplReplace = config.disableXtplReplace;

  var suffix = config.suffix;
  var entrySuffix = config.entrySuffix;
  var xtplSuffix = config.xtplSuffix;
  var otherSuffix = config.otherSuffix;
  var nativeSuffix = otherSuffix.native;
  var regCmdSuffix = config.regCmdSuffix;

  var regXtplSuffix = xtplSuffix ? new RegExp('\\'+xtplSuffix+'$') : false;

  // 最后生成的 depMap 对象
  var resultDepMap = {};

  // 如果是入口模块，如果有 entrySuffix 配置，需要拷贝一份
  if(entrySuffix && mainEntryModName){
    resultDepMap[mainEntryModName+entrySuffix] = depMap[mainEntryModName]; // 拷贝一份
  }

  // 如果不需要生成 xtpl 后缀的，直接返回
  if(disableXtplReplace){
    return resultDepMap;
  } 

  // 挨个遍历
  _.each(depMap, function(value, entryName){

    var newEntryName = '';
    var nativeDeps = []; // 存放新的native依赖，是以 .xtpl 结尾的
    // var nativeDepsOrigin = []; // 存放native依赖，仍然是以 .cmd 结尾的

    var isMainEntry = entryName === mainEntryModName; // 入口模块标志位

    // 后缀规则
    var canSuffix = util.canXtplSuffix(entryName);


    // === step 1 : 处理名字 index ->  index.xtpl
    // 将 .cmd 后缀修改成 .xtpl
    // 如果没有配置后缀名，则直接添加 .xtpl
    if(canSuffix){
      newEntryName = regCmdSuffix ? entryName.replace(regCmdSuffix, "") : entryName;
      newEntryName += xtplSuffix; // 统一添加后缀
    } else {
      newEntryName = entryName;
    }

    // debugExtra('=====> canSuffix:', canSuffix, ' newEntryName:', newEntryName);


    // === step 2: 过滤白名单，用户可以在 abc.json 中设置为 true，这样可以不过滤 native 白名单里
    var DepArrNoNative = config.disableBuiltIn ? value.requires : util.filterIgnore(value.requires, builtInList, 'origin-native');


    // === step 3: 处理依赖内部后缀
    if(canSuffix) {
      // 然后挨个 requires 数组元素，替换后缀名字
      _.each(DepArrNoNative, function(oriDep){

        var usedDep = oriDep;

        // 替换，将 .cmd -> .native.cmd
        var nativeValue = regCmdSuffix ? oriDep.replace(regCmdSuffix, "") : oriDep;
        nativeValue += nativeValue.nativeSuffix;

        // 检查原依赖数组中是否有native的依赖
        var isExistNative = _.has(depMap, nativeValue);

        // 如有，则使用native组件的依赖
        if(isExistNative) {
          usedDep = nativeValue;
        }

        // 如果是入口模块，则保存原始的 .cmd 依赖
        // 目前只有入口模块使用此逻辑，因为兼容所以才用到这个 nativeDepsOrigin 变量
        // if(isMainEntry){
        //   if(!config.disableXtplReplace){
        //     nativeDepsOrigin.push(usedDep);
        //   }
        // }

        // 更换 .cmd -> .xtpl
        var nativeXtpl = regCmdSuffix ? usedDep.replace(regCmdSuffix, "") : usedDep;
        nativeXtpl += xtplSuffix;

        // 将更改后的native依赖放入数组
        nativeDeps.push(nativeXtpl);

      });

      debugMini('====> 生成的native依赖：模块：',newEntryName,'；依赖：',nativeDeps);

      resultDepMap[newEntryName] = {requires: nativeDeps}; // 添加原始依赖

      // === start 兼容，需要拷贝一份入口文件，将名字修改成 .native
      if(isMainEntry){
        resultDepMap[mainEntryModName + '.native' + xtplSuffix] = {requires: nativeDeps};
      }
      // === end 兼容===
    } else {
      resultDepMap[newEntryName] = {
        requires: DepArrNoNative
      }
    }
  });

  return resultDepMap;
}

// 处理 web、native 依赖的后缀
function processOther(config){

  // 参数解构
  var depMap = config.depMap;
  var type = config.type;
  var mainEntryModName = config.mainEntryModName;
  var disableBuiltIn = config.disableBuiltIn;
  var disableXtplReplace = config.disableXtplReplace;

  var suffix = config.suffix;
  var entrySuffix = config.entrySuffix;
  var xtplSuffix = config.xtplSuffix;
  var otherSuffix = config.otherSuffix;
  var nativeSuffix = otherSuffix.native;
  var regCmdSuffix = config.regCmdSuffix;


  // 最后生成的 depMap 对象
  var resultDepMap = {};

  var newSuffix = otherSuffix[type];

  // 不存在对应的suffix的话，就直接返回
  if(!newSuffix){
  	return resultDepMap;
  }

  var newDeps = [];
  
  // 只针对入口模块做遍历
  if(mainEntryModName){

    var depArr = depMap[mainEntryModName] && depMap[mainEntryModName].requires || {}; // 获取require 数组

    // 如果是native的话，需要做一层过滤
    if(type === 'native'){
      depArr = config.disableBuiltIn ? depArr : util.filterIgnore(depArr, builtInList, config.disableUniqMods, 'native');
    }

	  // 然后挨个 requires 数组元素，替换后缀名字
    _.each(depArr, function(oriDep){

      var usedDep = oriDep;

      // 替换后缀 .cmd -> .native.cmd
      var newDepValue = regCmdSuffix ? oriDep.replace(regCmdSuffix, newSuffix) : oriDep + newSuffix;

      // 检查原依赖数组中是否有对应的依赖
      var isExistNew = _.has(depMap, newDepValue);

      // 如有，则使用新的依赖
      if(isExistNew) {
        usedDep = newDepValue;
      }

      // 将更改后的native依赖放入数组
      newDeps.push(usedDep);

    });

    // 构建新的入口名称
    var newEntryName = mainEntryModName + '.' + type + entrySuffix; // 给入口添加suffix
    
    resultDepMap[newEntryName] = {requires: newDeps}; // 添加原始依赖

    /** --兼容-- */
    // 如果是native的依赖，需要新增 .xtpl 后缀的入口
    if(type === 'native' && !disableXtplReplace){
      resultDepMap[newEntryName + xtplSuffix] = {
        requires: _.map(newDeps, function(dep){
          return regCmdSuffix ? dep.replace(regCmdSuffix, xtplSuffix) : dep+xtplSuffix;
        })
      }
    }
    
  }

  return resultDepMap;

}


// 根据类型返回新的 depMap 对象，不修改 depMap 的内容
// 入参：
//   depMap：最原始的 depMap 对象
// 	 mainEntryModName：主入口模块名，比如 'tb-countdown-1111/index'
// 	 type： 终端类型，目前有 origin，web，native 3种
// 	 config： 配置项
//
// return:
//   newDepMap：新的 depMap
module.exports = function(config) {

  // 参数解构
  var depMap = config.depMap;
  var type = config.type;
  var suffix = config.suffix;
  var entrySuffix = config.entrySuffix;
  var mainEntryModName = config.mainEntryModName;
  var disableXtplReplace = config.disableXtplReplace;
  var disableBuiltIn = config.disableBuiltIn;

  var xtplSuffix = '.xtpl'; // xtpl 后缀

  var otherSuffix = {
    'web': '.web' + suffix, // 组件所生成的 web 依赖后缀
    'native': '.native' + suffix // 组件所生成的 native 依赖后缀
  };

	var regCmdSuffix = suffix ? new RegExp('\\'+suffix+'$') : false;
  // var regCmdSuffix = /\.cmd/; // xtpl 的替换规则，没有 .cmd.xtpl 

  // 设置新的参数
  var newConfig = {
    depMap: depMap,
    type: type,
    suffix: suffix,
    entrySuffix: entrySuffix,
    mainEntryModName: mainEntryModName,
    disableXtplReplace: disableXtplReplace,
    disableBuiltIn: disableBuiltIn,
    xtplSuffix: xtplSuffix,
    otherSuffix: otherSuffix,
    regCmdSuffix: regCmdSuffix
  };

	if(type === 'origin'){
		return processOrigin(newConfig);
	} else {
		return processOther(newConfig);
	}

}