var assert = require('assert');
var proxyquire = require('proxyquire').noPreserveCache();
var path = require('path');
var fs = require('fs');
var Promise = require('bluebird');
var PATH_CASE = path.join(__dirname, 'cases');
var REG_ONLY = /^(only\-)/;
var mainEntry = path.join(__dirname, '../..', 'index');

/* global describe it */
var testCurrentCase = function(mainCase, testCase, isOnly) {
  return function() {
        // 获取测试用例文件夹
    var testDirectory = path.join(PATH_CASE, mainCase, testCase);
        // 模拟的数据
    var mockResponse = require(path.join(testDirectory, 'mock-res.js'));
        // 期待需要形成的deps.json
    var expectDeps = require(path.join(testDirectory, 'expect-json.js'));
        // 模拟的配置
    var config = require(path.join(testDirectory, 'config.js'));
        // 代理 upx-client 的响应数据
    var depsGen = proxyquire(mainEntry, {
      '@ali/upx-client': {
        upxClient: function() {
          return {
            'request': function() {
              return new Promise.resolve(mockResponse);
            }
          };
        }
      }
    }); // 测试index
        // promise 方式测试
    return depsGen(config).then(function(result) {
      if (isOnly) {
        console.log('=====> result.depMap:', JSON.stringify(result.depMap));
      }
      assert.deepEqual(result.depMap, expectDeps);
    });
  };
};
var casesDirs = fs.readdirSync(PATH_CASE);
// 主要测试用例
casesDirs.forEach(function(mainCase) {
    // 子套件测试用例，only功能
  var fn = REG_ONLY.test(mainCase) ? describe.only : describe;
  fn('index: ' + mainCase + ' 功能测试', () => {
        // 获取子测试用例名
    var subCasesDirs = fs.readdirSync(path.join(PATH_CASE, mainCase));
    subCasesDirs.forEach(function(testCase) {
      var caseName = [mainCase, testCase].join('-');
            // 增加only功能，方便单个功能测试
      var fn2 = REG_ONLY.test(testCase) ? it.only : it;
      fn2(caseName, testCurrentCase(mainCase, testCase, REG_ONLY.test(testCase)));
    });
  });
});