var fs = require('fs');
var path = require('path');
var assert = require('assert');
var webpack = require('webpack');
var Promise = require('bluebird');
var proxyquire =  require('proxyquire').noPreserveCache();


var PATH_CASE = path.join(__dirname, 'cases');
var PATH_CASE_RESULT = path.join(__dirname, 'output');
var REG_ONLY = /^(only\-)/;

var casesDirs = fs.readdirSync(PATH_CASE);
var pluginPath = path.join(__dirname, '../../plugin');

/* global describe it */

// 主要测试用例
casesDirs.forEach(mainCase => {

  var fn = REG_ONLY.test(mainCase) ? describe.only : describe;
  fn('plugin: ' + mainCase + ' 功能测试', () => {
    // 获取子测试用例名
    var subCasesDirs = fs.readdirSync(path.join(PATH_CASE, mainCase));

    subCasesDirs.forEach(testCase => {

      var caseName = [mainCase, testCase].join('-');

      // 增加only功能，方便单个功能测试
      var fn2 = REG_ONLY.test(testCase) ? it.only : it;
      fn2(caseName, testCurrentCase(mainCase, testCase, REG_ONLY.test(testCase)));
    });
  });
});


function testCurrentCase(mainCase, testCase, isOnly) {

  return ()=>{
    // 获取测试用例文件夹
    var testDirectory = path.join(PATH_CASE, mainCase, testCase);
    var outputDirectory = path.join(PATH_CASE_RESULT, mainCase, testCase);
    
    var abcOptions = require(path.join(testDirectory, 'abc.json'));
    var mockResult = require(path.join(testDirectory, 'mock-result.js'));
    abcOptions.options = {
      context: testDirectory
    };


    // 代理主函数的返回数据数据
    var DepsMapPlugin = proxyquire(pluginPath, {
      './index': ()=>{
        return new Promise.resolve(mockResult); // 返回模拟数据
      }
    }); 

    
    // 配置webpack运行参数
    var webpackConfig = {
      context: testDirectory,
      entry: {
        "index": "./index"
      },
      output:{
        filename:'[name].build.js',
        path: outputDirectory
      },
      plugins:[
        new DepsMapPlugin(abcOptions)
      ]
    };

    return new Promise((resolve, reject)=>{

      webpack(webpackConfig, (err, stats)=>{
        if(err) return reject(err);

        if(stats.hasErrors()) return reject(new Error(stats.toString()));

        assert.ok(true);
        resolve();
      })

    }).catch(err=>{
      console.log('eeeeerror:', err);
      assert.ok(false)
      reject();
    });
  }


}

